/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: DrawingContext.java,v 1.3 1996/06/03 16:34:31 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.image.ImageObserver;
import java.awt.Color;
import java.awt.Image;
import java.util.Stack;


public class DrawingContext {
    protected Component cpnt;
    protected Graphics grph;
    protected Rectangle alloc;
    
    private Stack styles = new Stack();

    public DrawingContext(Component target, Graphics graphics, 
			  Rectangle allocation, Style style) {
	cpnt = target;
	grph = graphics;
	alloc = allocation; 
	styles.push(style);
	style.apply(grph);
    }

    public DrawingContext(DrawingContext dc) {
	Rectangle r = dc.alloc;
	cpnt = dc.cpnt;
	grph = dc.grph;
	styles.push(dc.style());
	alloc = new Rectangle(r.x, r.y, r.width, r.height);
    }
    
    public Graphics graphics() {
	return grph;
    }
    
    public Rectangle allocation() {
	return alloc;
    }
    
    public Component target() {
	return cpnt;
    }
    
    public Dimension size() {
	return new Dimension(alloc.width, alloc.height);
    }
    
    public synchronized void setStyle(Style s) {
	if(style() == s) return;	// try to optimize things
	styles.pop();
	styles.push(s);
	s.apply(grph);
    }
    
    
    public synchronized void translate(int dx, int dy) {
	alloc.x += dx;
	alloc.y += dy;
    }
    
    public synchronized void move(int x, int y) {
	alloc.x = x;
	alloc.y = y;
    }
    
    public void resize(int w, int h) {
	alloc.resize(w, h);
    }
    
    public void drawString(String s, int x, int y) {
	grph.drawString(s, x+alloc.x, y+alloc.y);
    }
    
    public void drawRect(int x, int y, int w, int h) {
	grph.drawRect(x+alloc.x, y+alloc.y, w, h);
    } 
    
    public void fillPolygon(int[] x, int y[], int npoints) {
	for(int i = 0; i < npoints; i++) {
	    x[i] += alloc.x;
	    y[i] += alloc.y;
	}
	grph.fillPolygon(x, y, npoints);
    } 
    
    public boolean drawImage(Image img, int x, int y, ImageObserver observer) {
	return grph.drawImage(img, x+alloc.x, y+alloc.y, observer);
    }

    public void drawLine(int x1, int y1, int x2, int y2) {
	grph.drawLine(x1+alloc.x, y1+alloc.y, x2+alloc.x, y2+alloc.y);
    }
    
    public void drawRelief(int x, int y, int w, int h, int relief, int thick) {
	Relief.drawRect(grph, x+alloc.x, y+alloc.y, w, h, relief, thick);
    }
    
    public void fillRelief(int x, int y, int w, int h, int relief, int thick) {
	grph.fillRect(x+alloc.x+thick, y+alloc.y+thick, w-thick, h-thick);
	Relief.drawRect(grph, x+alloc.x, y+alloc.y, w, h, relief, thick);
    }
    
    public void fillRect(int x, int y, int w, int h) {
	grph.fillRect(x+alloc.x, y+alloc.y, w, h);
    }
    
    public void drawOvalShadow(int x, int y, int w, int h, 
			       int relief, int thick) {
	Relief.drawOval(grph, x+alloc.x, y+alloc.y, w, h, relief, thick);
    }
    
    public void fillOval(int x, int y, int w, int h) {
	grph.fillOval(x+alloc.x, y+alloc.y, w, h);
    }
    
    public void drawOval(int x, int y, int w, int h) {
	grph.drawOval(x+alloc.x, y+alloc.y, w, h);
    }

    public void drawRoundRect(int  x, int  y, int  w, int h,
			      int  arcWidth, int  arcHeight) {
	grph.drawRoundRect(x+alloc.x, y+alloc.y, w, h, arcWidth, arcHeight);
    }

    
    public void setColor(Color c) {
	grph.setColor(c);
    }
    
    public void vCenter(Look look) {
	int size = look.request(this).height;
	Dimension clip = size();
	int dh = (clip.height - size);
	
	translate(0, dh/2);
	resize(clip.width, clip.height - dh);
    }
    
    public void hCenter(Look look) {
	int size = look.request(this).width;
	Dimension clip = size();
	int dw = (clip.width - size);
	
	translate(dw/2, 0);
	resize(clip.width - dw, clip.height);
    }
    
    public void center(Look look) {
	Dimension dim = look.request(this);
	int width = dim.width;
	int height = dim.height;
	Dimension clip = size();
	int dw = (clip.width - width);
	int dh = (clip.height - height);
	
	translate(dw/2, dh/2);
	resize(clip.width - dw, clip.height - dh);
    }
    
    public void flushRight(Look look) {
	int w;
	Dimension clip = size();
	Dimension son = look.request(this);
	w = (clip.width - son.width);
	
	translate(w, 0);
	resize(clip.width - w,  clip.height);
    }
    
    public void shrink(int ldx, int tdy, int rdx, int bdy) {
	translate(ldx, tdy);
	resize(alloc.width - ldx - rdx, alloc.height - tdy - bdy);
    }
    
    public void shrinkTop(int dy) {
	translate(0, dy);
	resize(alloc.width, alloc.height - dy);
    }
    
    public void shrinkLeft(int dx) {
	translate(dx, 0);
	resize(alloc.width - dx, alloc.height);
    }
    
    public void shrinkRight(int dx) {
	resize(alloc.width - dx, alloc.height);
    }
    
    // Context manipulation methods
    
    public Style style() {
	return (Style)styles.peek();
    }
    
    public void push(Style s) {
	styles.push(s);
	s.apply(grph);
    }
    
    public Style pop() {
	Style old = (Style)styles.pop();
	style().apply(grph);
	return old;
    }
    
}
