/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id:$ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.gui;

import koala.util.*;

import java.awt.Rectangle;
import java.awt.Component;
import java.awt.Container;
import java.awt.Insets;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Dimension;
import java.util.Vector;


class BehaviorList extends Vector implements Behavior {
    
    public boolean handleEvent(Component c, Event evt) {
	boolean r = false;
	int n = size();
	for(int i = 0; i < n; i++) {
	    Behavior b = (Behavior) elementAt(i);
	    r |= b.handleEvent(c, evt);
	}
	return r;
    }
}



class LookList extends Vector implements Look {
    Dimension size;
    public void draw(DrawingContext dc) {
	int n = size();
	for(int i = 0; i < n; i++) {
	    Look l = (Look) elementAt(i);
	    l.draw(dc);
	}
    }
    
    public Dimension request(DrawingContext dc) {
	size = new Dimension(0, 0);
	int n = size();
	for(int i = 0; i < n; i++) {
	    Dimension r;
	    Look l = (Look) elementAt(i);
	    r = l.request(dc);
	    size.width  = Math.max(size.width, r.width);
	    size.height  = Math.max(size.height, r.height);
	}
	return size;
    }
    
    public Dimension size(DrawingContext dc) {
	return (size != null) ? size : request(dc);
    }
}


class CommandList extends Vector implements Command {
    public synchronized void execute(Component target, Event evt) {
	int n = size();
	for(int i = 0; i < n; i++) {
	    Command c = (Command) elementAt(i);
	    c.execute(target, evt);
	}
    }
}

/**  
 * this class is only used to group common behaviors that cannot be shared
 * between classes without multiple inheritance.  
 */
public class WidgetCode {

    public static Command onCommand(Command c, Command on) {
	if(c == null) {
	    return on;
	}
	CommandList cl;
	if(c instanceof CommandList) {
	    cl = (CommandList)c;
	    cl.addElement(on);
	}
	else {
	    cl = new CommandList();
	    cl.addElement(c);
	    cl.addElement(on);
	}
	return cl;
    }

    public static Command offCommand(Command c, Command off) {
	if(c == off) {
	    return null;
	}
	if(c instanceof CommandList) {
	    CommandList cl = (CommandList)c;
	    cl.removeElement(off);
	    if(cl.size() == 1) {
		// reduce vector to single element
		return ((Command)cl.elementAt(0));
	    }
	    else {
		return cl;
	    }
	}
	return c;
    }

    public static void appendBehavior(Widget w, Behavior b) {
	Behavior bv = w.getBehavior();
	if(bv == null) {
	    w.setBehavior(bv);
	}
	else {
	    if(bv instanceof BehaviorList) {
		BehaviorList bl = (BehaviorList)bv;
		bl.addElement(b);
	    }
	    else {
		BehaviorList bl = new BehaviorList();
		bl.addElement(bv);
		bl.addElement(b);
		w.setBehavior(bl);
	    }
	}
    }

    public static void prependBehavior(Widget w, Behavior b) {
	Behavior bv = w.getBehavior();
	if(bv == null) {
	    w.setBehavior(bv);
	}
	else {
	    if(bv instanceof BehaviorList) {
		BehaviorList bl = (BehaviorList)bv;
		bl.insertElementAt(b, 0);
	    }
	    else {
		BehaviorList bl = new BehaviorList();
		bl.addElement(b);
		bl.addElement(bv);
		w.setBehavior(bl);
	    }
	}
    }

    public static void removeBehavior(Widget w, Behavior b) {
	Behavior bv = w.getBehavior();
	if(bv == b) {
	    w.setBehavior(null);
	}
	else {
	    if(bv instanceof BehaviorList) {
		BehaviorList bl = (BehaviorList)bv;
		bl.removeElement(b);
		if(bl.size() == 1) {
		    // reduce vector to single element
		    w.setBehavior((Behavior)bl.elementAt(0));
		}
	    }
	}
    }

    public static void appendLook(Widget w, Look l) {
	Look lv = w.getLook();
	if(lv == null) {
	    w.setLook(lv);
	}
	else {
	    if(lv instanceof LookList) {
		LookList ll = (LookList)lv;
		ll.addElement(l);
	    }
	    else {
		LookList ll = new LookList();
		ll.addElement(lv);
		ll.addElement(l);
		w.setLook(ll);
	    }
	}
    }

    public static void prependLook(Widget w, Look l) {
	Look lv = w.getLook();
	if(lv == null) {
	    w.setLook(lv);
	}
	else {
	    if(lv instanceof LookList) {
		LookList ll = (LookList)lv;
		ll.insertElementAt(l, 0);
	    }
	    else {
		LookList ll = new LookList();
		ll.addElement(l);
		ll.addElement(lv);
		w.setLook(ll);
	    }
	}
    }

    public static void removeLook(Widget w, Look l) {
	Look lv = w.getLook();
	if(lv == l) {
	    w.setLook(null);
	}
	else {
	    if(lv instanceof LookList) {
		LookList ll = (LookList)lv;
		ll.removeElement(l);
		if(ll.size() == 1) {
		    // reduce vector to single element
		    w.setLook((Look)ll.elementAt(0));
		}
	    }
	}
    }

    static boolean handleEvent(Component c, Event e) {
	Widget w = (Widget)c;
	Behavior mgr = w.getBehavior();
	if((mgr != null) && (c.isEnabled())){
	    return mgr.handleEvent(c, e);
	}
	else {
	    return false;
	}
    }

    public static DrawingContext getDrawingContext(Widget w) {
	Component c = (Component)w;
	return new DrawingContext(c, c.getGraphics(), w.clipper(),
					w.getStyle());
    }
    
    static void paint(Component c, Widget w, Graphics g) {
	Look look = w.getLook();
	if(look != null) {
	    look.draw(new DrawingContext(c, g, w.clipper(),
					w.getStyle()));
	}
    }
    static final boolean dbuf =
    Boolean.getBoolean(Gui.DOUBLE_BUFFERING_P);
    static void update(Component c, Widget w, Graphics g, 
		      boolean oldDbufParam) {
	java.awt.Graphics grf = g;
	java.awt.Image img = null;
	Dimension d = c.size();
	
	if(dbuf) {
	    img = c.createImage(d.width, d.height);
	    grf = img.getGraphics();
	}
	// generic update
	grf.setColor(w.getStyle().background);
	grf.fillRect(0, 0, d.width, d.height);
	grf.setColor(w.getStyle().foreground);
	WidgetCode.paint(c, w, grf);
	
	if(dbuf) {
	    g.drawImage(img, 0, 0, c);
	}
    }

   /**
    * executes the command <code>cmd</code> if it is non-null, else
    * does nothing.
    */    
    public static void execute(Command cmd, Component c, Event e) {
	if(cmd != null) {
	    cmd.execute(c, e);
	}
    }
}
