/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: Listbox.java,v 1.10 1996/06/18 13:44:46 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.kit;

import koala.util.*;
import koala.gui.*;
import koala.look.*;

import java.awt.Event;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Rectangle;
import java.awt.Color;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;


public class Listbox extends Control implements Designation, Activation,
    Selection, Observer, Look, D2Scrollee {
    
    static protected Style defStyle = new Style("Listbox");

    protected int selectionPolicy = Selection.SINGLE;
    protected Adjustment hadjust;
    protected Adjustment vadjust;
    protected Vector datas;
    protected Vector items;
    protected SequenceModel selection;

   /**
    * the select command list.
    */	
    protected Command select = null;

   /**
    * the modified command list.
    */	
    protected Command designate = null;

   /**
    * the modified command list.
    */	
    protected Command modified = null;

   /**
    * the activate command list.
    */	
    protected Command activate = null;
    protected int visibleItemCount = 15;
    
    private Dimension size;
    private int first = 0;
    private int current = 0;
    
    public Listbox() {
	init(new ActiveSequence());
    }
    
    public Listbox(SequenceModel v) {
	init(v);
    }
    
    protected void init(SequenceModel v) {
	selection = v;
	datas = new Vector();
	items = new Vector();
	setLook(new BorderLook(this));
	setStyle(defStyle);
	setBehavior(new SelectBehavior(this));
	appendBehavior(new ActivateBehavior(this));
	selection.addObserver(this);
	hadjust = new ActiveAdjustment();
	vadjust = new ActiveAdjustment();
	hadjust.addObserver(this);
	vadjust.addObserver(this);
    }
    
    public synchronized int indexAt(int y) {
	DrawingContext dc = 
	  WidgetCode.getDrawingContext((Widget)this);
	int height = this.size().height;
	for(int i = first, ypos = 0; ypos <= height; i++) {
	    Look item = (Look)items.elementAt(i);
	    int h = item.size(dc).height;
	    if((y >= ypos) && (y < ypos + h)) {
		return i;
	    }
	    ypos += h;
	}
	return -1;
    }

    public void commit() {
	vadjust.set(vadjust.value(),
		    (float)visibleItemCount / (float)items.size());
	invalidate();
    }

    public void insert(Object o, Look i)  {
	datas.addElement(o);
	items.addElement(i);
    }
    
    public void insert(String l)  {
	datas.addElement(l);
	items.addElement(new VString(l));
    }

    public void removeAll() {
	selection.removeAll();
	datas.removeAllElements();
	items.removeAllElements();
    }

    float increment() {
	return 1.0f/(float)(items.size());
    }

    float pageIncrement() {
	return 10 * increment();	// sould be computed from contents
    }

    //============================================================
    //      implements: Selection
    //============================================================

   /**
    * Selects an object.
    *
    * Selects the specified item. If selection policy is SINGLE, deselects any
    * other selected item. Triggers the 'select' and 'modified' command lists.
    */
    public void select(Object item) {
	if(selectionPolicy == Selection.MULTIPLE) {
	    unselectAll(item);
	}
	selection.insert(item);
	WidgetCode.execute(select, this, null);
	WidgetCode.execute(modified, this, null);
    }
    
   /**
    * Selects all the items. Selection policy must be MULTIPLE.
    *
    * Triggers the 'select' and 'modified' command lists.
    */  
    public synchronized void selectAll() {
	if(selectionPolicy == Selection.SINGLE) {
	    throw new IllegalArgumentException("policy must be MULTIPLE");
	}
	selection.disable();
	selection.removeAll();
	for(int i = 0; i < datas.size(); i++) {
	    selection.insert(datas.elementAt(i));
	}
	selection.enable();
	WidgetCode.execute(select, this, null);
	WidgetCode.execute(modified, this, null);
    }
    
   /**
    * Unselects the specified item.
    *
    * Triggers the 'modified' command list.
    */    
    public void unselect(Object item) {
	selection.remove(item);
	WidgetCode.execute(modified, this, null);
    }
    
   /**
    * Unselects any selected item.
    *
    * Triggers the 'modified' command list.
    */    
    public void unselectAll(Object item) {
	selection.removeAll();
	WidgetCode.execute(modified, this, null);
    }

   /**
    * @return the current selection.
    */    
    public Enumeration selection() {
	return selection.elements();
    }
    
   /**
    * Sets selection policy.
    */
    public void setPolicy(int policy) {
	selectionPolicy = policy;
    }

   /**
    * Adds the specified command to the 'select' command list. The select
    * command list is triggered everytime a new item is selected.
    */
    public void onSelect(Command c) {
	select = WidgetCode.onCommand(select, c);
    }

   /**
    * Removes the specified command from the 'select' command list. The select
    * command list is triggered everytime a new item is selected.
    */
    public void offSelect(Command c) {
	select = WidgetCode.offCommand(select, c);
    }

   /**
    * Adds the specified command to the 'modified' command list. The select
    * command list is triggered everytime the selection changes.
    */
    public void onModified(Command c) {
	modified = WidgetCode.onCommand(modified, c);
    }

   /**
    * Removes the specified command from the 'modified' command list. The
    * select command list is triggered everytime the selection changes.
    */
    public void offModified(Command c) {
	modified = WidgetCode.offCommand(modified, c);
    }
    
    //============================================================
    //      implements: Designation
    //============================================================
    
    public void designate(Widget me, Event evt) {
	int n = indexAt(evt.y);
	if(n != -1) {
	    select(datas.elementAt(n));
	}
	WidgetCode.execute(designate, this, evt);
    }

   /**
    * Returns the current selection.
    */
    public Object current() {
	return null;
    }

   /**
    * Adds a Command to the designate command list.
    */    
    public Designation onDesignate(Command c) {
	designate = WidgetCode.onCommand(designate, c);
	return this;
    }


   /**
    * Removes a Command from the designate Command list.
    */    
    public Designation offDesignate(Command c) {
	designate = WidgetCode.offCommand(designate, c);
	return this;
    }

    //============================================================
    //      implements: Activation
    //============================================================

    
    public void activate(Widget me, Event evt) {
	WidgetCode.execute(activate, this, evt);
    }
   /**
    * Add a Command that is executed everytime an activation occurs.
    */    
    public void onActivate(Command c) {
	activate = WidgetCode.onCommand(activate, c);
    }

   /**
    * Remove a Command from the Command lis.
    */    
    public void offActivate(Command c) {
	activate = WidgetCode.offCommand(activate, c);
    }

    //============================================================
    //      implements: Observer
    //============================================================


    public void update(Observable o, Object arg) {
	repaint();
    }
    
    //============================================================
    //      implements: D2Scrollee
    //============================================================

    
    public void scrollForward(Adjustment adjust) {
	adjust.translate(increment(), 0f);
    }

    public void scrollBackward(Adjustment adjust) {
	adjust.translate(-increment(), 0f);
    }

    public void pageForward(Adjustment adjust) {
	adjust.translate(pageIncrement(), 0f);
    }

    public void pageBackward(Adjustment adjust) {
	adjust.translate(-pageIncrement(), 0f);
    }

    public void jumpTo(Adjustment adjust, float v) {
	
    }

    public Adjustment getHorizontalReferee() {
	return hadjust;
    }

    public Adjustment getVerticalReferee() {
	return vadjust;
    }

    
    //============================================================
    //      implements: Look
    //============================================================

    final static private int HMARGIN = 
      Integer.getInteger("Listbox.hmargin", 3).intValue();
    final static private Color SELECT_COLOR = 
      Color.getColor(Gui.SELECT_COLOR_P, Color.green);
    
    public void draw(DrawingContext dc) {
	Dimension itemsize;
	int last = items.size();
	Look item;
	first = (int)(items.size() * vadjust.value());
	DrawingContext subdc = new DrawingContext(dc);
	subdc.shrinkLeft(HMARGIN);
	subdc.shrinkRight(HMARGIN);
	for(int n = first; (n < last) && (subdc.size().height > 0); n++) {
	    item = (Look)items.elementAt(n);
	    itemsize = item.size(subdc);
	    if(selection.contains(datas.elementAt(n))) {
		// draw this Look as selected
		subdc.setColor(SELECT_COLOR);
		subdc.fillRelief(-HMARGIN, 0, subdc.size().width+HMARGIN*2-1, 
				 itemsize.height-1, Relief.RAISED, 1);
		subdc.setColor(subdc.style().foreground);
	    }
	    item.draw(subdc);
	    try {
		subdc.shrinkTop(itemsize.height);
	    }
	    catch (java.lang.NullPointerException e) {
		System.out.println(n);
		System.out.println(subdc);
		System.out.println(itemsize);
	    }
	}
    }
    
    public Dimension request(DrawingContext dc) {
	int width = 0, height = 0, h = 0, end = items.size();
	Dimension r;
	Look item;
	for(int i = first; (i < end) && (i < first+visibleItemCount); i++) {
	    item = (Look)items.elementAt(i);
	    r = item.request(dc);
	    width = Math.max(width, r.width);
	    height += r.height;
	}
	vadjust.set(vadjust.value(),
		    (float)visibleItemCount / (float)items.size());
	size = new Dimension(width+HMARGIN*2, height);
	return size;
    }
    
    public Dimension size(DrawingContext dc) {
	return (size != null) ? size : request(dc);
    }
    
}
