/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: BoxLayout.java,v 1.1 1996/06/13 14:30:48 leon Exp leon $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.misc;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Container;
import java.awt.LayoutManager;
import java.util.Hashtable;


/**
 * A BoxLayout-specific hint object.
 */
class BoxLayoutHint {

   /**
    * amount of extra space required of left side.
    */    
    int left;
    
   /**
    * amount of extra space required of right side.
    */   
    int right;
    
   /**
    * amount of extra space required above.
    */   
    int top;
    
   /**
    * amount of extra space required below.
    */   
    int bottom;
    
   /**
    * amount of extra space required for width.
    */   
    int width;
    
   /**
    * amount of extra space required for height.
    */   
    int height;

   /**
    * Creates a new BoxLayoutHint, with no extra space required.
    */    
    public BoxLayoutHint() {
	this(0, 0, 0, 0, 0, 0);
    }
    
   /**
    * Creates a new BoxLayoutHint, with specified extra space values.
    *
    * 
    */    
    public BoxLayoutHint(int left, int right, int top,
			       int bottom, int width, int height) {
	this.left = left;
	this.right = right;
	this.top = top;
	this.bottom = bottom;
	this.width = width;
	this.height = height;
    }
}


/**
 * A specialized Hashtable, just avoid dynamic casts outside this class.
 */
class BoxLayoutHintTable extends Hashtable {
    
    public BoxLayoutHint get(Component  key) {
	return (BoxLayoutHint)super.get(key);
    }
    
    public BoxLayoutHint put(Component key, BoxLayoutHint value) {
	return (BoxLayoutHint)super.put(key, value);
    }
    
    public BoxLayoutHint remove(Component key) {
	return (BoxLayoutHint)super.remove(key);
    }
}


/**
 * The BoxLayout, defines a genral geometry scheme, laying out components among
 * a major Axis. This major Axis is defined in the two subclasses HBoxLayout
 * and VBoxLayout. Main purpose of this layout is to specify dynamic geometry,
 * i.e it is poor to specify initial components layout, but is (I think) handy
 * to specify what happens when the parent container is resized.
 *
 * When no hint is specified, a BoxLayout simply places the components
 * along the major axis, keeping their preferred geometry.
 *
 * Given the parent's size, BoxLayoutContraint may be used to specify what
 * should happen to extra space on both axis. Extra space is simply distributed
 * to children that want them. Any children may require 6 differents extra
 * spaces: left, right, top, bottom, width, height. A BoxLayout first computes
 * the extra space required by the children and divides the total space
 * available proportionnally.  */
abstract public class BoxLayout implements LayoutManager {

    final static int GAP = 
      Integer.getInteger("HBoxLayout.gap", 0).intValue();
    final static int MAJORM = 
      Integer.getInteger("HBoxLayout.majorMargin", 0).intValue();
    final static int MINORM = 
      Integer.getInteger("HBoxLayout.minorMargin", 0).intValue();


    public final static int FIXED = 0;
    public final static int PROPORTIONAL = 1;
   /**
    * margin around each child for major axis
    */    
    int gap;

   /**
    * margin added along major 
    */    
    int majorm;

   /**
    * margin added along minor axis
    */    
    int minorm;

    BoxLayoutHintTable hints = new BoxLayoutHintTable();

    /**
     * Constructs a new BoxLayout, with default gap, major and minor margins.
     */
    public BoxLayout() {
	this(GAP, MAJORM, MINORM);
    }
    
    /**
     * Constructs a new BoxLayout, with specific gap, major and minor margins.
     *
     * @param gap Specifies gap.
     * @param maj specifies major margin.
     * @param min specifies minor margin.
     */
    public BoxLayout(int gap, int maj, int min) {
	gap = gap;
	majorm = maj;
	minorm = min;
    }

    public void addLayoutComponent(String name, Component comp) {
    }
    
    public void removeLayoutComponent(Component comp) {
	hints.remove(comp);
    }

    
    public Dimension minimumLayoutSize(Container target) {
	return preferredLayoutSize(target);
    }
     

   /**
    * Defines new location hints for the specified Component. It
    * overrides previous location hints for this component, but preserves
    * size hints.
    */
    public BoxLayout setLocationHints(Component c, int left, int right,
				       int top, int bottom) {
	BoxLayoutHint cs = hints.get(c);
	if(cs == null) {
	    cs = new BoxLayoutHint(left, right, top, bottom, 0, 0);
	    hints.put(c, cs);
	}
	else {
	    cs.left = left;
	    cs.right = right;
	    cs.top = top;
	    cs.bottom = bottom;
	}
	return this;
    }

   /**
    * defines new size hints for the specified Component. It overrides
    * previous size hints for this component, but preserves location
    * hints.
    */
    public BoxLayout setSizeHints(Component c, int width, int height) {
	BoxLayoutHint cs = hints.get(c);
	if(cs == null) {
	    cs = new BoxLayoutHint(0, 0, 0, 0, width, height);
	    hints.put(c, cs);
	}
	else {
	    cs.width = width;
	    cs.height = height ;
	}
	return this;
    }

    BoxLayoutHint getHints(Component c) {
	BoxLayoutHint cs = hints.get(c);
	if(cs == null) {
	    cs = new BoxLayoutHint(0, 0, 0, 0, 0, 0);
	    hints.put(c, cs);
	}
	return cs;
    }

    public BoxLayout setLeftHint(Component c, int v) {
	getHints(c).left = v;
	return this;
    }
    public BoxLayout setRightHint(Component c, int v) {
	getHints(c).right = v;
	return this;
    }
    
    public BoxLayout setTopHint(Component c, int v) {
	getHints(c).top = v;
	return this;
    }
    
    public BoxLayout setBottomHint(Component c, int v) {
	getHints(c).bottom = v;
	return this;
    }
    
    public BoxLayout setWidthHint(Component c, int v) {
	getHints(c).width = v;
	return this;
    }
    
    public BoxLayout setHeightHint(Component c, int v) {
	getHints(c).height = v;
	return this;
    }
}     
