/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: ActiveAdjustment.java,v 1.3 1996/06/18 16:12:20 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.util;

/**
 * An Active implementation of Adjustment that notifies its changes to its
 * observers
 */
public class ActiveAdjustment extends Subject implements Adjustment {
    static final float MAXRANGE = 1.0f;
    static final float MINRANGE = 0.0f;
    protected float lbound;
    protected float hbound;
    protected float value;
    protected float fraction;


   /**
    * Builds a new Adjustment with specified value and fraction.
    *
    * @param v current value.
    * @param f current fraction
    */
    public ActiveAdjustment(float v, float f) {
	lbound = MINRANGE;
	hbound = MAXRANGE;
	value = v;
	fraction = f;
    }

   /**
    * Builds a new Adjustment with value set to MINRANGE and fraction set to
    * MAXRANGE.
    */
    public ActiveAdjustment() {
	this(MINRANGE, MAXRANGE);
    }

    //============================================================
    //      implements: Adjustment
    //============================================================

   /**
    * Sets new values to the Adjustment. Triggers the update method for
    * attached observers.
    *
    * @param v The new value.
    * @param f The new fraction.
    */
    public void set(float v, float f) {
	if(v < lbound) {
	    v = lbound;
	}
	else if(v > hbound) {
	    v = hbound;
	}
	if(f < MINRANGE) {
	    f = MINRANGE;
	}
	else if (f > MAXRANGE) {
	    f = MAXRANGE;
	}
	if(v != value) {
	    value = v;
	    setChanged();
	}
	if(f != fraction) {
	    fraction = f;
	    setChanged();
	}
	notifyObservers();
    }

   /**
    * Adds the specified values as value and fraction. Triggers the update
    * method for attached observers.
    *
    * @param dv The delta added to the value: final value is constrained to
    * stay between the bounds
    *
    * @param df The delta added to the fraction: final fraction is constrained
    * to stay between (high bound - low bound) */
    public void translate(float dv, float df) {
	set(value + dv, fraction + df);
    }

   /**
    * Returns the current value.
    */    
    public float value() {
	return value;
    }

   /**
    * Returns the current fraction.
    */    
    public float fraction() {
	return fraction;
    }

    public String toString() {
	return getClass().getName() + "[value=" + value +
	    ",fraction=" + fraction + "]";
    }
}
