/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: App.java,v 1.4 1996/06/18 16:12:23 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.util;

import java.util.Properties;
import java.io.*;

class Default {
    static String properties = 
      "gui.imagesPath=images" + "\n" +
      "gui.background=0xC0C0C0" + "\n" +
      "gui.foreground=0x000000" + "\n" +
      "gui.font=Helvetica-bold-12" + "\n" +
      "gui.checkColor=0xFFFF00" + "\n" +
      "gui.selectColor=0x00cd00" + "\n" +
      "Scale.foreground=0xC0C0C0" + "\n" ;
}

/**
 * provides some usefull Application-level functionalities
 */
public class App {

   /**
    * When true, enables verbose messages printing.
    * Initial value is computed from the \'verbose\' property, so it can be 
    * set at startup with "java -Dverbose=true".
    */
    static public final boolean VERBOSE = Boolean.getBoolean("verbose");

   /**
    * When true, enables debug messages printing.
    * Initial value is computd from the \'debug\' property, so it can be 
    * set at startup with "java -Ddebug=true".
    */
    static public final boolean DEBUG   = Boolean.getBoolean("debug");

   /**
    * Loads a properties file in the current application's database.
    * if the <code>profile.dir</code> property is set, it is used as local 
    * directory to search the profile, else the profile is searched in the 
    * user's home directory. If the property file is not found, nothing 
    * happens.
    *
    * @param name The name of the profile to be loaded.
    */    
    static public void loadProfile(String name) {
	Properties props;
	try {
	    props = System.getProperties();

	    String profile;
	    String prodir = System.getProperty("profile.home");
	    if(prodir == null) {
		prodir = System.getProperty("user.home");
	    }
	    profile = prodir + System.getProperty("file.separator") + name;
	    verbose("Loading properties from: " + profile);
	    try {
		props.load(new FileInputStream(profile));
	    }
	    catch(FileNotFoundException e) {
		// no property file, keep silent
		verbose("Cannot find properties");
	    }
	    catch(IOException e) {
		verbose("Error while reading profile: " + profile);
	    }
	}
	catch(SecurityException e) {
	    props = new Properties();
	    verbose("Loading default properties.");
	    try {
		props.load(new StringBufferInputStream(Default.properties));
	    }
	    catch(IOException ioe) {
		verbose("Error while reading default properties");
	    }
	}
    }

   /**
    * Outputs a message, prefixed with "Warning"
    */
    static public void warning(String msg) {
	System.err.println("Warning: " + msg);
    }

   /**
    * Outputs a message, depending on the global 'verbose' flag
    * @see #VERBOSE
    */
    static public void verbose(String msg) {
	if(VERBOSE) {
	    System.err.println(msg);
	}
    }

   /**
    * Outputs a message, depending on the global 'debug' flag.
    * @see #DEBUG
    */
    static public void debug(String msg) {
	if(DEBUG) {
	    System.err.println(msg);
	}
    }

   /**
    * When debug mode is on, asserts that object <code>o</code> is not null. 
    * If it raises a NullPointerException.
    * @see #DEBUG
    */    
    static public final void assert(Object o) {
	// hope that a compiler could optimize this, in nothing.
	if(DEBUG) {
	    if(o == null) {
		throw new NullPointerException("assertion failed");
	    }
	}
    }

   /**
    * When debug mode is on, asserts that condition <code>o</code> is true. 
    * If not it raises a RuntimeException.
    * @see #DEBUG
    */  
    static public final void assert(boolean cond) {
	// hope that a compiler could optimize this, in nothing.
	if(DEBUG) {
	    if(! cond) {
		throw new RuntimeException("condition failed");
	    }
	}
    }
    
    static public String getStringProp(String name, String def) {
	try {
	    return System.getProperty(name, def);
	}
	catch(SecurityException e) {
	    return def;
	}
    }
}
