/* Copyright (c) 1996 by Groupe Bull.  All Rights Reserved */
/* $Id: MutableString.java,v 1.3 1996/06/18 16:12:24 leon Exp $ */
/* Author: Jean-Michel.Leon@sophia.inria.fr */

package koala.util;


/**
 * Another implementation of the StringBuffer class, with delete methods.
 */
public class MutableString {
    private int length = 0;
    private int capacity;
    private char buffer[];

   /**
    * Builds a new empty MutableString.
    */
    public MutableString() {
	this(16);
    }

   /**
    * Builds a new MutableString, with value set to 'init'.
    */
    public MutableString(String init) {
	this();
	insert(0, init);
    }

   /**
    * Builds a new empty MutableString with room for 'n' chars.
    */    
    public MutableString(int n) {
	buffer = new char[n];
	capacity = n;
    }

   /**
    * Returns the length of the string value.
    */    
    public int length() {
	return length;
    }

   /**
    * Returns the current String value.
    */
    public String string() {
	return new String(buffer, 0, length);
    }

   /**
    * Returns the String formed with 'n' characters, starting at 'offset'.
    */
    public String string(int offset, int n) {
	checkIndex(offset);
	checkIndex(offset+n);
	return new String(buffer, offset, n);
    }

    private final void checkIndex(int pos) {
	if(pos > length)
	    throw new IllegalArgumentException("index '" + pos + "' too high");
	if(pos < 0)
	    throw new IllegalArgumentException("index '" + pos + "' too low");
    }
    
    private synchronized void ensureCapacity(int n) {
	if(capacity >= n) return;
	char newbuffer[] = new char[n];
	System.arraycopy(buffer, 0, newbuffer, 0, length);
	buffer = newbuffer;
	capacity = n;
    }

   /**
    * inserts character 'c' at position 'pos'.
    */    
    public synchronized MutableString insert(int pos, char c) {
	checkIndex(pos);
	
	ensureCapacity(length+1);
	System.arraycopy(buffer, pos, buffer, pos+1, length - pos);
	buffer[pos] = c;
	length++;
	return this;
    }

   /**
    * Inserts String s, starting at position 'pos'.
    */    
    public synchronized MutableString insert(int pos, String s) {
	checkIndex(pos);
	
	int l = s.length();
	char a[] = s.toCharArray();
	ensureCapacity(length+s.length());
	System.arraycopy(buffer, pos, buffer, pos+l, length - pos);
	System.arraycopy(a, 0, buffer, pos, l);
	length += l;
	return this;
    }

   /**
    * Deletes character at position 'pos'.
    */    
    public synchronized MutableString delete(int pos) {
	checkIndex(pos);
	if(pos < length) {
	    System.arraycopy(buffer, pos+1, buffer, pos, length - pos - 1);
	}
	length--;
	return this;
    }

   /**
    * Deletes 'n' characters, starting at position 'pos'.
    */    
    public synchronized MutableString delete(int pos, int n) {
	checkIndex(pos);
	
	System.arraycopy(buffer, pos+n, buffer, pos, length - pos);
	length -= n;
	return this;
    }

    public String toString() {
	return getClass().getName() + "[cap=" + capacity + ",len=" + length +
	    ",\"" + new String(buffer, 0, length) + "\"]";
    }
}
